#=======================================================================
#	$Id: QMapped.pm,v 1.3 2007/04/21 16:33:31 pythontech Exp $
#	Triple store in which URIs are qmapped
#	Copyright (C) 2006-2007  Python Technology Limited
#
#	This program is free software; you can redistribute it and/or
#	modify it under the terms of the GNU General Public License
#	as published by the Free Software Foundation; either version 2
#	of the License, or (at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
#	02111-1307, USA.
#-----------------------------------------------------------------------
#	In the underlying triplestore, URIs are stored as qnames.
#	This module provides a wrapper by which full URIs are seen.
#=======================================================================
package PythonTech::Triples::QMapped;
use strict;
use PythonTech::Triples;

our @ISA = qw(PythonTech::Triples);

#-----------------------------------------------------------------------
#	Create a new Triples::QMapped.
#	  qmap		QMap
#	  qtriples	Triple store whose URIs are q-encoded
#-----------------------------------------------------------------------
sub new {
    my $class = shift;
    my $self = {@_};
    foreach (qw(qmap qtriples)) {
	die $class.": $_ not configured" 
	    unless defined $self->{$_};
    }
    bless $self, $class;
    return $self;
}

sub iterate {
    my($self, $xs,$xp,$xo,$cb) = @_;
    my $map = $self->{'qmap'};
    my @spo = map {
	(! defined $_ ||
	 substr($_,0,1) eq '=') ?
	     $_ :
		 $map->encode($_);
    } $xs,$xp,$xo;
#    print STDERR "iterate: ",join(" ",@spo),"\n";
    my $res = $self->{'qtriples'}->iterate(@spo, sub {
	my @xspo = map {
	    substr($_,0,1) eq '=' ?
		$_ :
		    $map->decode($_);
	} @_;
#	print STDERR " cb: ",join(" ",@xspo),"\n";
	my $r = $cb->(@xspo);
	$r;
    });
    return $res;
}

sub add {
    my($self, $xs,$xp,$xo) = @_;
    my $map = $self->{'qmap'};
    my $s = $map->encode($xs);
    my $p = $map->encode($xp);
    my $o = substr($xo,0,1) eq '=' ?
	$xo : $map->encode($xo);
    $self->{'qtriples'}->add($s,$p,$o);
}

sub remove {
    my($self, $xs,$xp,$xo) = @_;
    my $map = $self->{'qmap'};
    my $s = $map->encode($xs);
    my $p = $map->encode($xp);
    my $o = substr($xo,0,1) eq '=' ?
	$xo : $map->encode($xo);
    $self->{'qtriples'}->remove($s,$p,$o);
}

sub commit {
    shift->{'qtriples'}->commit;
}

sub rollback {
    shift->{'qtriples'}->rollback;
}

sub disconnect {
    shift->{'qtriples'}->disconnect;
}

sub reconnect {
    shift->{'qtriples'}->reconnect(@_);
}

sub refresh {
    my($self, @args) = @_;
    return unless $self->{'qtriples'}->can('refresh');
    $self->{'qtriples'}->refresh(@args);
}

1;
